import { EventEmitter, Component, Renderer2, ElementRef, Input, ContentChildren, ViewChild, HostListener, NgModule } from '@angular/core';
import { isUndefined, isString, isNumber } from 'util';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Simple class to help the creation of bidi binding attributes.
 * As typescript and angular doesn't enfore types this class helps to handle it.
 *
 * Instead of:
 * ```ts
 * \@Output('valueChange') value: EventEmitter<number> = new EventEmitter<number>();
 * \@Input('value') value: number = 1; // parsing code removed
 * ```
 *
 * ```ts
 * _value = new Attribute<number>(this.yChange, NUMBER_PARSER, 1);
 * \@Output('valueChange') valueChange = _value.eventEmitter;
 * \@Input('value') set value(value: number) { this._value.value = value; }
 * get value(): number {return this._value.value; }
 * ```
 *
 * @see https://github.com/angular/components/tree/master/src/cdk/coercion
 * @template T
 */
/* tslint:disable:variable-name no-output-rename no-input-rename curly*/
class Attribute {
    /**
     * @param {?} parser
     * @param {?=} value
     */
    constructor(parser, value) {
        this._event = new EventEmitter();
        this._parser = parser;
        if (!isUndefined(value)) {
            this._value = value;
        }
    }
    /**
     * @return {?}
     */
    get eventEmitter() { return this._event; }
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (this._value !== v) {
            v = this._parser(v);
            if (this._value !== v) {
                this._value = v;
                this._event.emit(this._value);
            }
        }
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @return {?}
     */
    complete() {
        this._event.complete();
    }
    /**
     * @return {?}
     */
    toString() {
        return typeof this.value + ' ' + this.value;
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    Attribute.prototype._parser;
    /**
     * @type {?}
     * @private
     */
    Attribute.prototype._event;
    /**
     * @type {?}
     * @private
     */
    Attribute.prototype._value;
}
/** @type {?} */
const NOP_PARSER = (/**
 * @param {?} source
 * @return {?}
 */
(source) => {
    return source;
});
/** @type {?} */
const STRING_PARSER = (/**
 * @param {?} source
 * @return {?}
 */
(source) => {
    if (source == null)
        return null;
    if (isString(source))
        return source;
    return '' + source;
});
/**
 * Converts the given value to a number
 * https://flaviocopes.com/how-to-convert-string-to-number-javascript/
 * \@param source input value
 * @type {?}
 */
const NUMBER_PARSER = (/**
 * @param {?=} source
 * @return {?}
 */
(source) => {
    if (source == null)
        return null;
    if (isNumber(source))
        return source;
    /** @type {?} */
    const result = source * 1;
    if (isNaN(result))
        return null;
    return result;
});
/**
 * Like the NUMBER_PARSER but always returns integers.
 * \@param source any value to parse
 * @type {?}
 */
const INT_PARSER = (/**
 * @param {?} source
 * @return {?}
 */
(source) => {
    source = NUMBER_PARSER(source);
    if (source == null)
        return null;
    return Math.round(source);
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function NgStaticGridItemModel() { }
if (false) {
    /** @type {?} */
    NgStaticGridItemModel.prototype.x;
    /** @type {?} */
    NgStaticGridItemModel.prototype.y;
    /** @type {?} */
    NgStaticGridItemModel.prototype.width;
    /** @type {?} */
    NgStaticGridItemModel.prototype.height;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Any item can have x and y coordinates and a width and hight.
 *
 * x / y are 0 based.
 */
/* tslint:disable:curly*/
class NgStaticGridItemComponent {
    /**
     * @param {?} renderer
     * @param {?} hostElement
     */
    constructor(renderer, hostElement) {
        this.renderer = renderer;
        this.hostElement = hostElement;
        this.x = 0;
        this.y = 0;
        this.width = 1;
        this.height = 1;
        renderer.addClass(hostElement.nativeElement, 'static-grid-item');
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.width = NUMBER_PARSER(this.width) || 1;
        this.height = NUMBER_PARSER(this.height) || 1;
        this.x = NUMBER_PARSER(this.x) || 0;
        this.y = NUMBER_PARSER(this.y) || 0;
    }
    /**
     * Applies the given model to the attributes
     * @param {?} val
     * @return {?}
     */
    setModel(val) {
        this.x = val.x;
        this.y = val.y;
        this.height = val.height;
        this.width = val.width;
        this.ngOnInit(); // just to parse the attributes
    }
    /**
     * Builds the model from the current attributes.
     * @return {?}
     */
    getModel() {
        return {
            height: this.height, width: this.width,
            x: this.x, y: this.y
        };
    }
    /**
     * @param {?} width
     * @param {?} height
     * @return {?}
     */
    doPosition(width, height) {
        this.renderer.setStyle(this.hostElement.nativeElement, 'width', (width * this.width) + '%');
        this.renderer.setStyle(this.hostElement.nativeElement, 'height', (height * this.height) + '%');
        this.renderer.setStyle(this.hostElement.nativeElement, 'left', (width * this.x) + '%');
        this.renderer.setStyle(this.hostElement.nativeElement, 'top', (height * this.y) + '%');
    }
}
NgStaticGridItemComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-static-grid-item',
                template: `<ng-content></ng-content>`,
                styles: [`
  :host {
    position: absolute;
  }`]
            }] }
];
/** @nocollapse */
NgStaticGridItemComponent.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef }
];
NgStaticGridItemComponent.propDecorators = {
    id: [{ type: Input }],
    x: [{ type: Input }],
    y: [{ type: Input }],
    width: [{ type: Input }],
    height: [{ type: Input }]
};
if (false) {
    /**
     * To save and restore the postion each items needs an ID to be identified.
     * @type {?}
     */
    NgStaticGridItemComponent.prototype.id;
    /** @type {?} */
    NgStaticGridItemComponent.prototype.x;
    /** @type {?} */
    NgStaticGridItemComponent.prototype.y;
    /** @type {?} */
    NgStaticGridItemComponent.prototype.width;
    /** @type {?} */
    NgStaticGridItemComponent.prototype.height;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridItemComponent.prototype.renderer;
    /** @type {?} */
    NgStaticGridItemComponent.prototype.hostElement;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function NgStaticGridModel() { }
if (false) {
    /** @type {?} */
    NgStaticGridModel.prototype.rows;
    /** @type {?} */
    NgStaticGridModel.prototype.columns;
    /** @type {?} */
    NgStaticGridModel.prototype.width;
    /** @type {?} */
    NgStaticGridModel.prototype.height;
    /** @type {?} */
    NgStaticGridModel.prototype.items;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:curly*/
class NgStaticGridPanelComponent {
    /**
     * @param {?} hostElement
     */
    constructor(hostElement) {
        this.hostElement = hostElement;
        this.rows = 12;
        this.columns = 12;
        this.width = '100%';
        this.height = '100%';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.rows = INT_PARSER(this.rows);
        this.columns = INT_PARSER(this.columns);
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        this.doPosition();
    }
    /**
     * @return {?}
     */
    getModel() {
        /** @type {?} */
        const result = {
            width: this.width,
            height: this.height,
            columns: this.columns,
            rows: this.rows,
            items: new Map()
        };
        if (this.items) {
            this.items.forEach((/**
             * @param {?} i
             * @return {?}
             */
            i => {
                if (i.id)
                    result.items[i.id] = i.getModel();
            }));
        }
        return result;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setModel(val) {
        this.rows = INT_PARSER(val.rows) || 12;
        this.columns = INT_PARSER(val.columns) || 12;
        this.width = val.width;
        this.height = val.height;
        this.items.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (val.items && val.items[item.id]) {
                item.setModel(val.items[item.id]);
            }
        }));
    }
    /**
     * Position all elements in the view
     * @return {?}
     */
    doPosition() {
        /** @type {?} */
        const w = 100 / this.columns;
        /** @type {?} */
        const h = 100 / this.rows;
        this.items.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            item.doPosition(w, h);
        }));
    }
}
NgStaticGridPanelComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-static-grid-panel',
                template: `
  <div class="static-grid"
    [style.width]="width"
    [style.height]="height">
    <ng-content></ng-content>
  </div>
  `,
                styles: [`
  .static-grid {
    position: relative;
  }`]
            }] }
];
/** @nocollapse */
NgStaticGridPanelComponent.ctorParameters = () => [
    { type: ElementRef }
];
NgStaticGridPanelComponent.propDecorators = {
    items: [{ type: ContentChildren, args: [NgStaticGridItemComponent,] }],
    rows: [{ type: Input }],
    columns: [{ type: Input }],
    width: [{ type: Input }],
    height: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    NgStaticGridPanelComponent.prototype.items;
    /** @type {?} */
    NgStaticGridPanelComponent.prototype.rows;
    /** @type {?} */
    NgStaticGridPanelComponent.prototype.columns;
    /** @type {?} */
    NgStaticGridPanelComponent.prototype.width;
    /** @type {?} */
    NgStaticGridPanelComponent.prototype.height;
    /** @type {?} */
    NgStaticGridPanelComponent.prototype.hostElement;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const LineType = {
    U_TYPE: 'u-type',
    S_TYPE: 's-type',
};
/* tslint:disable:curly variable-name*/
class NgStaticGridCanvasComponent {
    /**
     * @param {?} hostElement
     */
    constructor(hostElement) {
        this.hostElement = hostElement;
        /**
         * Optional color of the shadow.
         * \@optional default #343a40
         */
        this.shadowColor = '#343a40';
        /**
         * Position of the error head
         * \@values bottom | top
         * \@memberof NgStaticGridCanvasComponent
         */
        this.arrowHead = 'bottom';
        this._contentSet = false;
        this._reversed = false;
        this._lineType = LineType.U_TYPE;
        this._strokeStyle = '#ae2424';
        this._gridSizeX = 24;
        this._gridSizeY = 24;
    }
    /**
     * @return {?}
     */
    get reversed() {
        return this._reversed;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set reversed(value) {
        this._reversed = '' + value !== 'false';
    }
    /**
     * @param {?} type
     * @return {?}
     */
    set lineType(type) {
        switch (type) {
            case LineType.U_TYPE:
                this._lineType = type;
                break;
            case LineType.S_TYPE:
                this._lineType = type;
                break;
            default:
                this._lineType = LineType.U_TYPE;
        }
    }
    /**
     * @return {?}
     */
    get strokeStyle() {
        return this._strokeStyle;
    }
    /**
     * Set the color of the line.
     * @see https://www.w3schools.com/colors/colors_picker.asp
     * @param {?} val
     * @return {?}
     */
    set strokeStyle(val) {
        if (this._strokeStyle !== val) {
            this._strokeStyle = val;
            this.doRedraw();
        }
    }
    /**
     * @return {?}
     */
    get gridSizeX() {
        return this._gridSizeX;
    }
    /**
     * Int number how many columns
     * \@memberof NgStaticGridCanvasComponent
     * @param {?} val
     * @return {?}
     */
    set gridSizeX(val) {
        this._gridSizeX = INT_PARSER(val) || 24;
    }
    /**
     * @return {?}
     */
    get columns() {
        return this.gridSizeX;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set columns(val) {
        this.gridSizeX = val;
    }
    /**
     * @return {?}
     */
    get gridSizeY() {
        return this._gridSizeY;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set gridSizeY(val) {
        this._gridSizeY = INT_PARSER(val) || 24;
    }
    /**
     * @return {?}
     */
    get rows() {
        return this.gridSizeY;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set rows(val) {
        this.gridSizeY = val;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._xFactor = Math.floor(this.gridSizeX / 12);
        this._yFactor = Math.floor(this.gridSizeY / 12);
        if (this.strokeGridFactor == null) {
            this.strokeGridFactor = Math.min(this._xFactor, this._yFactor, 1);
        }
        if (this.gridStartX == null) {
            this.gridStartX = this._xFactor;
        }
        if (this.gridEndX == null) {
            this.gridEndX = this.gridSizeX - this.gridStartX;
        }
        if (this.gridStartY == null) {
            this.gridStartY = this._yFactor;
        }
        if (this.gridEndY == null) {
            this.gridEndY = this.gridSizeY - this.gridStartY;
        }
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        // select the parent if possible, if no element to cover was set
        if (!this.coverElement) {
            if (this.hostElement.nativeElement.parentNode) {
                this.coverElement = new ElementRef(this.hostElement.nativeElement.parentNode);
            }
            else {
                this.coverElement = this.hostElement;
            }
        } // else console.info('coverElement set ...', this.coverElement);
        this._contentSet = true;
        this.doRedraw();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onResize(event) {
        this.adjustCanvas(event.target.innerHeight, event.target.innerWidth);
    }
    /**
     * @return {?}
     */
    doRedraw() {
        if (window && this._contentSet) {
            this.adjustCanvas(window.innerHeight, window.innerWidth);
            // console.info('doRedraw ...');
        }
    }
    /**
     * @param {?} height
     * @param {?} width
     * @return {?}
     */
    adjustCanvas(height, width) {
        if (this.canvas) {
            /** @type {?} */
            const rect = this.coverElement.nativeElement.getBoundingClientRect();
            // console.info('adjustCanvas', rect, width, height, width - rect.left, height - rect.top);
            this.canvas.nativeElement.height = height - rect.top || 0;
            this.canvas.nativeElement.width = width - rect.left || 0;
            this.prepareLine();
        }
    }
    /**
     * @return {?}
     */
    prepareLine() {
        if (this.canvas && this.canvas.nativeElement
            && this.canvas.nativeElement.getContext('2d')) {
            /** @type {?} */
            const canvas = this.canvas.nativeElement;
            /** @type {?} */
            const context = canvas.getContext('2d');
            // https://www.w3schools.com/graphics/canvas_gradients.asp
            context.strokeStyle = this.strokeStyle;
            context.fillStyle = this.strokeStyle;
            if (this.shadowColor)
                context.shadowColor = this.shadowColor;
            if (this.alpha)
                context.globalAlpha = NUMBER_PARSER(this.alpha);
            /** @type {?} */
            const oneX = canvas.width / this.gridSizeX;
            // one step in X
            /** @type {?} */
            const oneY = canvas.height / this.gridSizeY;
            context.lineWidth = Math.min(oneX, oneY) * this.strokeGridFactor;
            if (this.shadowColor)
                context.shadowBlur = context.lineWidth / this.strokeGridFactor;
            /** @type {?} */
            const startX = oneX * this.gridStartX;
            /** @type {?} */
            const startY = oneY * this.gridStartY;
            /** @type {?} */
            const endX = oneX * this.gridEndX;
            /** @type {?} */
            const endY = oneY * this.gridEndY;
            // console.info(this.gridStartX, this.gridStartY, this.gridEndX, this.gridEndY);
            // console.info(startX, startY, endX, endY);
            switch (this._lineType) {
                case LineType.U_TYPE:
                    if (this.reversed) {
                        this.drawReversedULine(context, canvas, startX, endX, startY, endY, oneX, oneY);
                    }
                    else {
                        this.drawULine(context, canvas, startX, endX, startY, endY, oneX, oneY);
                    }
                    break;
                case LineType.S_TYPE:
                    if (this.reversed) {
                        this.drawReversedSLine(context, canvas, startX, endX, startY, endY, oneX, oneY);
                    }
                    else {
                        this.drawSLine(context, canvas, startX, endX, startY, endY, oneX, oneY);
                    }
                    break;
            }
        }
    }
    /**
     * @private
     * @param {?} context
     * @param {?} canvas
     * @param {?} startX
     * @param {?} endX
     * @param {?} startY
     * @param {?} endY
     * @param {?} oneX
     * @param {?} oneY
     * @return {?}
     */
    drawReversedSLine(context, canvas, startX, endX, startY, endY, oneX, oneY) {
        context.clearRect(0, 0, canvas.width, canvas.height);
        /** @type {?} */
        const lineX = oneX * this.strokeGridFactor;
        /** @type {?} */
        const lineY = oneY * this.strokeGridFactor;
        /** @type {?} */
        const radius = 40;
        /** @type {?} */
        const curvesAmount = 2;
        /** @type {?} */
        const curveStep = (endY - startY) / curvesAmount;
        context.lineWidth = 1;
        context.beginPath();
        if (this.arrowHead === 'bottom') {
            context.moveTo(endX, startY);
            context.lineTo(endX, startY + lineY);
        }
        else {
            context.moveTo(endX - lineY, startY);
            this.drawCanvasArrow(context, endX, startY, lineX, lineY, 'right');
        }
        // first inner u-turn
        this.drawCurve(context, startX + lineY, startY + lineY, radius, 'bottom', 'left');
        this.drawCurve(context, startX + lineY, startY + curveStep, radius, 'right', 'left');
        // first u-turn
        this.drawCurve(context, endX, startY + curveStep, radius, 'bottom', 'right', lineY);
        this.drawCurve(context, endX, endY + lineY, radius, 'left', 'right', lineY);
        if (this.arrowHead === 'bottom') {
            this.drawCanvasArrow(context, startX, endY, lineX, lineY, 'left');
        }
        else {
            context.lineTo(startX, endY + lineY);
            context.lineTo(startX, endY);
        }
        // second inner u-turn
        this.drawCurve(context, endX - lineY, endY, radius, 'top', 'left');
        this.drawCurve(context, endX - lineY, startY + curveStep + lineY, radius, 'left', 'left');
        // second u-turn
        this.drawCurve(context, startX, startY + curveStep + lineY, radius, 'top', 'right', lineY);
        this.drawCurve(context, startX, startY, radius, 'right', 'right', lineY);
        context.closePath();
        context.fill();
    }
    /**
     * @private
     * @param {?} context
     * @param {?} canvas
     * @param {?} startX
     * @param {?} endX
     * @param {?} startY
     * @param {?} endY
     * @param {?} oneX
     * @param {?} oneY
     * @return {?}
     */
    drawSLine(context, canvas, startX, endX, startY, endY, oneX, oneY) {
        context.clearRect(0, 0, canvas.width, canvas.height);
        /** @type {?} */
        const lineX = oneX * this.strokeGridFactor;
        /** @type {?} */
        const lineY = oneY * this.strokeGridFactor;
        /** @type {?} */
        const radius = 40;
        /** @type {?} */
        const curvesAmount = 2;
        /** @type {?} */
        const curveStep = (endY - startY) / curvesAmount;
        context.lineWidth = 1;
        context.beginPath();
        if (this.arrowHead === 'bottom') {
            context.moveTo(startX, startY + lineY);
            context.lineTo(startX, startY);
        }
        else {
            context.moveTo(startX + lineX, startY + lineY);
            this.drawCanvasArrow(context, startX, startY, lineX, lineY, 'left');
        }
        // first u-turn
        this.drawCurve(context, endX, startY, radius, 'bottom', 'right', lineY);
        this.drawCurve(context, endX, startY + curveStep + lineY, radius, 'left', 'right', lineY);
        // first inner u-turn
        this.drawCurve(context, startX + lineY, startY + curveStep + lineY, radius, 'bottom', 'left');
        this.drawCurve(context, startX + lineY, endY, radius, 'right', 'left');
        if (this.arrowHead === 'bottom') {
            this.drawCanvasArrow(context, endX, endY, lineX, lineY, 'right');
        }
        else {
            context.lineTo(endX, endY);
            context.lineTo(endX, endY + lineY);
        }
        // second u-turn
        this.drawCurve(context, startX, endY + lineY, radius, 'top', 'right', lineY);
        this.drawCurve(context, startX, startY + curveStep, radius, 'right', 'right', lineY);
        // second inner u-turn
        this.drawCurve(context, endX - lineY, startY + curveStep, radius, 'top', 'left');
        this.drawCurve(context, endX - lineY, startY + lineY, radius, 'left', 'left');
        context.closePath();
        context.fill();
    }
    /**
     * @private
     * @param {?} context
     * @param {?} canvas
     * @param {?} startX
     * @param {?} endX
     * @param {?} startY
     * @param {?} endY
     * @param {?} oneX
     * @param {?} oneY
     * @return {?}
     */
    drawULine(context, canvas, startX, endX, startY, endY, oneX, oneY) {
        context.clearRect(0, 0, canvas.width, canvas.height);
        /** @type {?} */
        const lineX = oneX * this.strokeGridFactor;
        /** @type {?} */
        const lineY = oneY * this.strokeGridFactor;
        /** @type {?} */
        const radius = 40;
        context.lineWidth = 1;
        // ---->
        context.beginPath();
        if (this.arrowHead === 'bottom') {
            context.moveTo(startX, startY + lineY);
            context.lineTo(startX, startY);
        }
        else {
            this.drawCanvasArrow(context, startX, startY, lineX, lineY, 'left');
        }
        // first u-turn
        this.drawCurve(context, endX, startY, radius, 'bottom', 'right', lineY);
        this.drawCurve(context, endX, endY + lineY, radius, 'left', 'right', lineY);
        // arrow head bottom
        if (this.arrowHead === 'bottom') {
            this.drawCanvasArrow(context, startX, endY, lineX, lineY, 'left');
        }
        else {
            context.lineTo(startX, endY + lineY);
            context.lineTo(startX, endY);
        }
        // first inner u-turn
        this.drawCurve(context, endX - lineY, endY, radius, 'top', 'left');
        this.drawCurve(context, endX - lineY, startY + lineY, radius, 'left', 'left');
        context.closePath();
        context.fill();
    }
    /**
     * @private
     * @param {?} context
     * @param {?} canvas
     * @param {?} startX
     * @param {?} endX
     * @param {?} startY
     * @param {?} endY
     * @param {?} oneX
     * @param {?} oneY
     * @return {?}
     */
    drawReversedULine(context, canvas, startX, endX, startY, endY, oneX, oneY) {
        context.clearRect(0, 0, canvas.width, canvas.height);
        /** @type {?} */
        const lineX = oneX * this.strokeGridFactor;
        /** @type {?} */
        const lineY = oneY * this.strokeGridFactor;
        /** @type {?} */
        const radius = 40;
        context.lineWidth = 1;
        // ---->
        context.beginPath();
        if (this.arrowHead === 'bottom') {
            context.moveTo(endX, startY);
            context.lineTo(endX, startY + lineY);
        }
        else {
            this.drawCanvasArrow(context, endX, startY, lineX, lineY, 'right');
        }
        // first inner u-turn
        this.drawCurve(context, startX + lineY, startY + lineY, radius, 'bottom', 'left');
        this.drawCurve(context, startX + lineY, endY, radius, 'right', 'left');
        // arrow head bottom
        if (this.arrowHead === 'bottom') {
            this.drawCanvasArrow(context, endX, endY, lineX, lineY, 'right');
        }
        else {
            context.lineTo(endX, endY);
            context.lineTo(endX, endY + lineY);
        }
        // first u-turn
        this.drawCurve(context, startX, endY + lineY, radius, 'top', 'right', lineY);
        this.drawCurve(context, startX, startY, radius, 'right', 'right', lineY);
        context.closePath();
        context.fill();
    }
    /**
     * A method to draw an arrow on the canvas
     * @private
     * @param {?} context is the canvas context
     * @param {?} leftArrowPointX is the X coordinate of the Point which is located on the left bottom part of the arrow
     * @param {?} leftArrowPointY is the Y coordinate of the Point which is located on the left bottom part of the arrow
     * @param {?} lineWidthX is X coordinate Width of the arrow
     * @param {?} lineWidthY is Y coordinate Width of the arrow
     * @param {?} direction is the direction in which the arrow should point to
     * @return {?}
     */
    drawCanvasArrow(context, leftArrowPointX, leftArrowPointY, lineWidthX, lineWidthY, direction) {
        if (direction === 'left') {
            context.lineTo(leftArrowPointX + lineWidthX, leftArrowPointY + lineWidthY);
            context.lineTo(leftArrowPointX + lineWidthX, leftArrowPointY + lineWidthY + (lineWidthY / 2));
            context.lineTo(leftArrowPointX, leftArrowPointY + lineWidthY / 2);
            context.lineTo(leftArrowPointX + lineWidthX, leftArrowPointY - lineWidthY / 2);
            context.lineTo(leftArrowPointX + lineWidthX, leftArrowPointY);
        }
        if (direction === 'right') {
            context.lineTo(leftArrowPointX - lineWidthX, leftArrowPointY);
            context.lineTo(leftArrowPointX - lineWidthX, leftArrowPointY - lineWidthY / 2);
            context.lineTo(leftArrowPointX, leftArrowPointY + lineWidthY / 2);
            context.lineTo(leftArrowPointX - lineWidthX, leftArrowPointY + lineWidthY + (lineWidthY / 2));
            context.lineTo(leftArrowPointX - lineWidthX, leftArrowPointY + lineWidthY);
        }
    }
    /**
     * A Method to draw from a specific corner a curve
     * @private
     * @param {?} context is the canvas context
     * @param {?} cornerX is the X coordinate of the corner
     * @param {?} cornerY is the Y coordinate of the corner
     * @param {?} radius is the radius of the curve
     * @param {?} direction is the direction in which the final curve should point to
     * @param {?} curve specifies if it should be a left or a right curve
     * @param {?=} lineWidth for outer line curves the lineWidth is needed to adjust the radius
     * @return {?}
     */
    drawCurve(context, cornerX, cornerY, radius, direction, curve, lineWidth) {
        if (lineWidth) {
            radius = radius + lineWidth;
        }
        /** @type {?} */
        let startX = cornerX;
        /** @type {?} */
        let startY = cornerY;
        /** @type {?} */
        let endX = cornerX;
        /** @type {?} */
        let endY = cornerY;
        switch (direction) {
            case 'top':
                if (curve === 'right') {
                    startX = startX + radius;
                    endY = endY - radius;
                }
                else {
                    startX = startX - radius;
                    endY = endY - radius;
                }
                break;
            case 'right':
                if (curve === 'right') {
                    startY = startY + radius;
                    endX = endX + radius;
                }
                else {
                    startY = startY - radius;
                    endX = endX + radius;
                }
                break;
            case 'bottom':
                if (curve === 'right') {
                    startX = startX - radius;
                    endY = endY + radius;
                }
                else {
                    startX = startX + radius;
                    endY = endY + radius;
                }
                break;
            case 'left':
                if (curve === 'right') {
                    startY = startY - radius;
                    endX = endX - radius;
                }
                else {
                    startY = startY + radius;
                    endX = endX - radius;
                }
                break;
        }
        context.lineTo(startX, startY);
        context.quadraticCurveTo(cornerX, cornerY, endX, endY);
    }
}
NgStaticGridCanvasComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-static-grid-canvas',
                template: `
        <canvas #canvas class="canvas">
        </canvas>
    `,
                styles: [":host{position:absolute;margin:0;padding:0;display:-webkit-box;display:flex}.canvas{margin:0;padding:0}"]
            }] }
];
/** @nocollapse */
NgStaticGridCanvasComponent.ctorParameters = () => [
    { type: ElementRef }
];
NgStaticGridCanvasComponent.propDecorators = {
    canvas: [{ type: ViewChild, args: ['canvas', { static: true },] }],
    coverElement: [{ type: Input }],
    shadowColor: [{ type: Input }],
    alpha: [{ type: Input }],
    gridStartX: [{ type: Input }],
    gridEndX: [{ type: Input }],
    gridStartY: [{ type: Input }],
    gridEndY: [{ type: Input }],
    strokeGridFactor: [{ type: Input }],
    arrowHead: [{ type: Input }],
    reversed: [{ type: Input, args: ['reversed',] }],
    lineType: [{ type: Input }],
    strokeStyle: [{ type: Input }],
    gridSizeX: [{ type: Input }],
    columns: [{ type: Input }],
    gridSizeY: [{ type: Input }],
    rows: [{ type: Input }],
    onResize: [{ type: HostListener, args: ['window:resize', ['$event'],] }]
};
if (false) {
    /** @type {?} */
    NgStaticGridCanvasComponent.prototype.canvas;
    /**
     * The element to cover
     * @type {?}
     */
    NgStaticGridCanvasComponent.prototype.coverElement;
    /**
     * Optional color of the shadow.
     * \@optional default #343a40
     * @type {?}
     */
    NgStaticGridCanvasComponent.prototype.shadowColor;
    /**
     * Optional global alpha to set.
     * @type {?}
     */
    NgStaticGridCanvasComponent.prototype.alpha;
    /** @type {?} */
    NgStaticGridCanvasComponent.prototype.gridStartX;
    /** @type {?} */
    NgStaticGridCanvasComponent.prototype.gridEndX;
    /** @type {?} */
    NgStaticGridCanvasComponent.prototype.gridStartY;
    /** @type {?} */
    NgStaticGridCanvasComponent.prototype.gridEndY;
    /**
     * Takes the min of width or hight and mutiplies it with this number.
     * So that the line scales with the available screen size.
     * @type {?}
     */
    NgStaticGridCanvasComponent.prototype.strokeGridFactor;
    /**
     * Position of the error head
     * \@values bottom | top
     * \@memberof NgStaticGridCanvasComponent
     * @type {?}
     */
    NgStaticGridCanvasComponent.prototype.arrowHead;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._xFactor;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._yFactor;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._contentSet;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._reversed;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._lineType;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._strokeStyle;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._gridSizeX;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype._gridSizeY;
    /**
     * @type {?}
     * @private
     */
    NgStaticGridCanvasComponent.prototype.hostElement;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgStaticGridModule {
}
NgStaticGridModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgStaticGridItemComponent, NgStaticGridPanelComponent, NgStaticGridCanvasComponent],
                imports: [],
                exports: [NgStaticGridItemComponent, NgStaticGridPanelComponent, NgStaticGridCanvasComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { Attribute, INT_PARSER, NOP_PARSER, NUMBER_PARSER, NgStaticGridCanvasComponent, NgStaticGridItemComponent, NgStaticGridModule, NgStaticGridPanelComponent, STRING_PARSER };
//# sourceMappingURL=ng-static-grid.js.map
